@@ -1,4 +1,4 @@
-VERSION = '0.0.3'
+VERSION = '0.0.8'
 
 import collections
 from os import path
@@ -43,11 +43,14 @@ class IniLoader(Loader):
 		config = ConfigParser()
 		config.read(cfile)
 
-		if not self.with_profile:
-			return config.defaults()
-
 		ret = {sec: dict(config.items(sec)) for sec in config.sections()}
-		ret['default'] = config.defaults()
+		defaults = config.defaults()
+		defaults.update(ret.get('default', {}))
+		ret['default'] = defaults
+		
+		if not self.with_profile:
+			return defaults
+
 		return ret
 
 class EnvLoader(Loader):
@@ -173,46 +176,61 @@ Loaders = dict(
 	dict   = DictLoader
 )
 
+class NoSuchProfile(Exception):
+	pass
+
 class Config(ConfigBox):
 	
-	def __init__(self, with_profile = True, case_sensitive = False, *args, **kwargs):
-		super(Config, self).__init__(*args, **kwargs)
+	def __init__(self, *args, **kwargs):
 		self.__dict__['_protected'] = dict(
-			with_profile   = with_profile,
-			case_sensitive = case_sensitive,
+			with_profile   = kwargs.pop('with_profile', True),
+			case_sensitive = kwargs.pop('case_sensitive', False),
 			profile        = 'default',
 			cached         = OrderedDict()
 		)
+		super(Config, self).__init__(*args, **kwargs)
 
 	def get(self, key, default = None, cast = None):
 		ret = super(Config, self).get(key, default)
 		return cast(ret) if callable(cast) else ret
 
 	def _load(self, *names):
+		cached         = self._protected['cached']
+		with_profile   = self._protected['with_profile']
+		case_sensitive = self._protected['case_sensitive']
+		profile        = self._protected['profile']
 		for name in names:
 			ext = 'dict' if isinstance(name, dict) else name.rpartition('.')[2]
 			if ext not in Loaders:
 				raise FormatNotSupported(ext)
 			if ext == 'dict':
-				if repr(name) not in self._protected['cached']:
-					self._protected['cached'][repr(name)] = DictLoader(name, self._protected['with_profile'], self._protected['case_sensitive']).config
+				if repr(name) not in cached:
+					cached[repr(name)] = DictLoader(name, with_profile, case_sensitive).config
 				name = repr(name)
 			else:
-				if name not in self._protected['cached']:
-					self._protected['cached'][name] = Loaders[ext](name, self._protected['with_profile'], self._protected['case_sensitive']).config
+				# maybe hash the name?
+				if name not in cached:
+					cached[name] = Loaders[ext](name, with_profile, case_sensitive).config
+				else:
+					# change the position of the configuration
+					cached[name] = cached.pop(name)
 
-			if self._protected['with_profile']:
-				self.update(self._protected['cached'][name].get(self._protected['profile'].upper(), {}))
+			if with_profile:
+				self.update(cached[name].get(profile if case_sensitive else profile.upper(), {}))
 			else:
-				self.update(self._protected['cached'][name])
+				self.update(cached[name])
 
 	def copy(self, profile = 'default'):
-		ret = self.__class__(self._protected['with_profile'], self._protected['case_sensitive'], **self)
+		ret = self.__class__(with_profile = self._protected['with_profile'], case_sensitive = self._protected['case_sensitive'], **self)
 		ret.__dict__['_protected']['profile'] = profile
 		ret.__dict__['_protected']['cached']  = self._protected['cached']
 		return ret
+
+	def clear(self):
+		super(Config, self).clear()
+		self._protected['cached'] = OrderedDict()
 	
-	def _use(self, profile = 'default'):
+	def _use(self, profile = 'default', raise_exc = False):
 		if not self._protected['with_profile']:
 			raise ValueError('Unable to switch profile, this configuration is set without profile.')
 
@@ -224,10 +242,10 @@ class Config(ConfigBox):
 
 		if self._protected['case_sensitive']:
 			if self._protected['profile'] != 'default':
-				self.clear()
+				super(Config, self).clear()
 		else:
 			if self._protected['profile'].upper() != 'DEFAULT':
-				self.clear()
+				super(Config, self).clear()
 
 		if self._protected['case_sensitive']:
 			if profile != 'default':
@@ -238,8 +256,15 @@ class Config(ConfigBox):
 				# load default first
 				self._use()
 		
+		hasprofile = False
 		for conf in self._protected['cached'].values():
-			self.update(conf.get(profile, {}))
+			if profile not in conf:
+				continue
+			hasprofile = True
+			self.update(conf[profile])
+			
+		if raise_exc and not hasprofile:
+			raise NoSuchProfile('Config has no such profile: %s' % profile)
 
 		self._protected['profile'] = profile
 
